.global main             @ int main(void)
.global _etext           @    ROM( FLASH )
.global _data            @    RAM
.global _edata           @     RAM
.global __bss_start      @    (bss-)   RAM
.global __bss_end__      @  bss
.global _stack           @   

@    
        .set  UND_STACK_SIZE, 0x00000004
        .set  ABT_STACK_SIZE, 0x00000004
		.set  FIQ_STACK_SIZE, 0x00000004
        .set  IRQ_STACK_SIZE, 0X00000080
        .set  SVC_STACK_SIZE, 0x00000004


        .set  MODE_USR, 0x10            @  USER
        .set  MODE_FIQ, 0x11            @   FIQ
        .set  MODE_IRQ, 0x12            @  IRQ
        .set  MODE_SVC, 0x13            @  Supervisor
        .set  MODE_ABT, 0x17            @  Abort
        .set  MODE_UND, 0x1B            @   Undefined
        .set  MODE_SYS, 0x1F            @  System

        .equ  I_BIT, 0x80               @   I    IRQ
        .equ  F_BIT, 0x40               @   F   FIQ

 .text
	.arm
	.section .init, "ax"

        .code 32
        .align 2

        .global _boot
        .func   _boot
_boot:
Vectors:
        b     _start                    @ reset - _start
        ldr   pc,_undf                  @ undefined - _undf
        ldr   pc,_swi                   @ SWI - _swi
        ldr   pc,_pabt                  @ program abort - _pabt
        ldr   pc,_dabt                  @ data abort - _dabt
        nop                             @ reserved
        ldr   pc,[pc,#-0xFF0]           @ IRQ - read the VIC
        ldr   pc,_fiq                   @ FIQ - _fiq

_undf:  .word __undf                    @ undefined
_swi:   .word __swi                     @ SWI
_pabt:  .word __pabt                    @ program abort
_dabt:  .word __dabt                    @ data abort
_irq:   .word __irq                     @ IRQ
_fiq:   .word __fiq                     @ FIQ

__undf: b     .                         @ undefined
__swi:  b     .                         @ SWI
__pabt: b     .                         @ program abort
__dabt: b     .                         @ data abort
__irq:  b     .                         @ IRQ
__fiq:  b     .                         @ FIQ
.size _boot, . - _boot	@    _boot       _boot
.endfunc		@  _boot

   .global _start, start, _mainCRTStartup
        .func   _start

_start:
start:
_mainCRTStartup:

        ldr   r0,=_stack

		msr   CPSR_c,#MODE_UND|I_BIT|F_BIT @ Undefined Instruction Mode
        mov   sp,r0
        sub   r0,r0,#UND_STACK_SIZE

		msr   CPSR_c,#MODE_ABT|I_BIT|F_BIT @ Abort Mode
        mov   sp,r0
        sub   r0,r0,#ABT_STACK_SIZE


		msr   CPSR_c,#MODE_FIQ|I_BIT|F_BIT @ FIQ Mode
        mov   sp,r0
        sub   r0,r0,#FIQ_STACK_SIZE


        msr   CPSR_c,#MODE_IRQ|I_BIT|F_BIT @ IRQ Mode
        mov   sp,r0
        sub   r0,r0,#IRQ_STACK_SIZE


        msr   CPSR_c,#MODE_SVC|I_BIT|F_BIT @ Supervisor Mode
        mov   sp,r0
        sub   r0,r0,#SVC_STACK_SIZE


        msr   CPSR_c,#MODE_SYS|I_BIT|F_BIT @ System Mode
        mov   sp,r0


       ldr   r1,=_etext                @ -> ROM data start
        ldr   r2,=_data                 @ -> data start
        ldr   r3,=_edata                @ -> end of data
1:      cmp   r2,r3                     @ check if data to move
        ldrlo r0,[r1],#4                @ copy it
        strlo r0,[r2],#4
        blo   1b                        @ loop until done

       mov   r0,#0                     @ get a zero
        ldr   r1,=__bss_start           @ -> bss start
        ldr   r2,=__bss_end__           @ -> bss end
2:      cmp   r1,r2                     @ check if data to clear
        strlo r0,[r1],#4                @ clear 4 bytes
        blo   2b                        @ loop until done
		    mov   r0,#0                     @ no arguments (argc = 0)
        mov   r1,r0
        mov   r2,r0
        mov   fp,r0                     @ null frame pointer
        mov   r7,r0                     @ null frame pointer for thumb
        ldr   r10,=main
        mov   lr,pc
         bx    r10                       @ enter main()
		 .size   _start, . - _start
        .endfunc

        .global _reset, reset, exit, abort
        .func   _reset
_reset:
reset:
exit:
abort:
		b 	_reset 				@loop until reset

        .size _reset, . - _reset
        .endfunc

        .end
